//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#ifndef BLOCK_H_INCLUDED_
#define BLOCK_H_INCLUDED_

class StoneBoard;

// class represents a set of strongly connected points, black or white;
// stone blocks are investigated for liberties and surrounding, while empty blocks aren't
class Block: public StoneRegion
{
public:
    
    Block( int id, int color, const Point& start, BoardNavigator* bnav );
    virtual ~Block();
    
    virtual void preprocess();
    virtual void update_props();
            
    void AnalyseStatus();
    int  CountInternalDame( StoneBoard* terri );
    
    int  BBSquare() const { return (bbR - bbL) * (bbB - bbT); }
    bool IsSafe() const { return predicted_status > 0.5; }
    int  SharedLiberties( const Block& block ) const;
    //int  CountAdjacentDame( const StoneBoard& terri );

    int chainID;

    IndexArray adjESpaces;
    IndexArray adjESpaceLiberties;

    double predicted_status;    // NN input/output (associative feedback)
    int    expert_estimate;     // life/death status, as predicted by expert
    bool   in_question;         // whether predicted_status is based on static analysis

    PointArray friends;         // stones of the same color, that share a liberty with block
    PointArray liberties[3];    // three levels of liberties for stone blocks
    PointArray connections;     // chain connection points (empty)
    //PointArray cutfriends;
    //PointArray cutpoints;       // opponent points, cutting two blocks apart
    int protected_liberties;    // protected liberties of the block
    int autoatari_liberties;    // auto-atari liberties of hte block
    int local_majority;         // opponents - friendly stones within manhattan distance of 2
    int cm_x, cm_y;             // centre-of-mass of the block

    CERBEArray cerEntries;

    // blocks properties for feature vector for neural network
    int eye_num,
        eye_sum_size,
        eye_sum_perimeter,
        eye_sum_split_points;

    double
        pacer_average_power;

    int pacer_num,
        pacer_sum_acc_size,
        pacer_sum_acc_perimeter,
        pacer_sum_acc_split_points,
        pacer_sum_int_size,
        pacer_sum_int_perimeter,
        pacer_sum_int_split_points;

    int opp_cer_size,
        opp_cer_perimeter;

    int terr_direct_liberties,
        terr_all_friend_liberties,
        terr_all_opp_liberties;

    int opp_w1, opp_w2;         // weakest and second weakest opponent blocks
    int fri_s1;                 // strongest friendly block with shared liberties

private:

    void ExtendBlock( const Point& start );
    void AnalyseGeometry();
    void CountSpecialLiberties();   // protected and auto-atari
    void FindWeakestAndStrongest();
    void ExamineAdjCERs();
    
    int bbL, bbT, bbR, bbB;
};


typedef std::vector<Block*> BlockArray;


#endif