//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#ifndef CER_H_INCLUDED_
#define CER_H_INCLUDED_

class IndexBoard;
class MarkerBoard;

// color-enclosed region - set of strongly connected points, surrounded by the given color
// CERs are investigated for eyespace and eyeshape
class CER: public Region
{
public:

    CER( int id, int border, const Point& start, BoardNavigator* bnav );
    virtual ~CER() {}

    virtual void preprocess() {}
    virtual void update_props();

    PointArray eye_points;  // if opponents == 0, region is an eye, so this is full eyepoints
    int eye_perimeter;      // full eyepoints perimeter
    int eye_split_points;   // number of split points in the eye
    int opponents;          // number of the opponents stones in the region
    double average_power;   // average influence value over CER; friendly to the border!

private:

    void Extend( const Point& pos, IndexBoard* index, MarkerBoard* temp );
    void ExamineEyeShape();   // find all full eyepoints
    void CalcAveragePower();
};


// CER objects storage
typedef std::vector<CER*> CERArray;


// this class represents some numerical features of the color-enclosed region,
// adjacent to the given block
class CER_Block_Entry
{
public:
    
    CER_Block_Entry( const CER& cer, const Block& block );

    PointArray acc_points;  // all empty points, accessible from the block
    int acc_perimeter;      // perimeter of acc_points set
    int acc_split_points;   // number of split points in acc_points set

    PointArray int_points;    // all interior points unaccessible from given block
    int int_perimeter;
    int int_split_points;
};


typedef std::vector<CER_Block_Entry*> CERBEArray;


#endif // CER_H_INCLUDED_