//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#ifndef CHAIN_H_INCLUDED_
#define CHAIN_H_INCLUDED_


class Chain: public StoneRegion
{
public:

    Chain( int id, Block* start, BoardNavigator* bnav );
    virtual ~Chain() {}

    virtual void preprocess();
    virtual void update_props();
    
    void ExaminePerimeter();
    void CalcAverageStatus();
    double GetMinimalStatus();
    
    IndexArray blocks;
    IndexArray shaOppBlocks;        // opponent blocks, which share at least one liberty with chain
    PointArray extOppLiberties;     // external opponent liberties

    PointArray liberties;
    PointArray territory;
    PointArray split_liberties;
    
    int eye_num;
    int eye_sum_size;
    int eye_sum_perimeter;
    int eye_sum_split_points;
    
    int     pacer5_num;
    double  pacer5_avepow;
    int     pacer9_num;
    double  pacer9_avepow;

    int opp_w3;         // weakest opponent block, adjacent to or sharing liberties with
    double average_status;

private:

    void Extend( Block* block );
    void CountExternalOppLiberties();
    void FindClosestOppBlocks(); // opponents blocks that share liberties with chain
    void FindWeakestOppBlock();
    void CountSplitLiberties();
    void CheckEyespace();
};


typedef std::vector<Chain*> ChainArray;


#endif CHAIN_H_INCLUDED_