//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#include "StdAfx.h"
#include "ESpace.h"
#include "..\\Navigator\\BoardNavigator.h"

ESpace::ESpace( int id, const Point& start, BoardNavigator* bnav )
: Region( id, eEmpty, bnav )
{
    Extend( start );
}


// PURPOSE: extends the region from given point with flood filling
void ESpace::Extend( const Point& pos )
{
    // add point to the block, index and mark it
    points.push_back(pos);
    navi->blip.nc_set(pos, id());

    int pi = 0;
    while ( pi < points.size() )
    {
        Point p( points[pi] );

        for ( int n = 0; n < 4; n++ )
        {
            Point adj( p.adjacent(n) );
            if ( !adj.ok() )
            {
                perimeter_++; 
                continue;
            }

            if ( navi->board.nc_empty(adj) && navi->blip.nc_get(adj) == -1 )
            {
                points.push_back(adj);
                navi->blip.nc_set(adj, id());
            } else if ( !navi->board.nc_empty(adj) )
            {
                neibours.push_back(adj);
            }
        }
        pi++;
    }

    neibours.remove_duplicates();
    perimeter_ += neibours.size();
}


// PURPOSE: looks for adjacent black and white blocks, counts their liberties
void ESpace::preprocess()
{
    MarkerArray id_marker( navi->allRegions.size() );
    int whites_ = 0;
    int blacks_ = 0;
    in_question = false;

    MarkerBoard point_marker;
    point_marker.nc_mark( points );

    for ( int ni = neibours.size(); ni-- > 0; )
    {
        Point p( neibours[ni] );
        if ( !p.ok() ) continue;

        Block* block = navi->block_at(p);
        if ( id_marker.marked( block->id() ) )
            continue;

        id_marker.mark( block->id() );

        if ( block->color() == eBlack ) blacks_++;
        if ( block->color() == eWhite ) whites_++;

        int liberties = 0;
        for ( int pi = block->liberties[0].size(); pi-- > 0; )
        {
            if ( point_marker.nc_marked( block->liberties[0][pi] ) )
                liberties ++;
        }

        // make cross reference
        adjBlocks.push_back( block->id() );
        adjBlocksLiberties.push_back( liberties );

        block->adjESpaces.push_back( id() );
        block->adjESpaceLiberties.push_back( liberties );
    }

    if ( blacks_ && whites_ ) in_question = true;
}
