//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#include "StdAfx.h"
#include "Point.h"
#include <functional>


Point Point::adjacent_[cBoardSize][4];
Point Point::diagonal_[cBoardSize][4];
PointShiftCache Point::cache_;


PointShiftCache::PointShiftCache()
{
    for ( int pos = 0; pos < cBoardSize; pos++ )
        for ( int n = 0; n < 4; n++ )
        {
            Point::adjacent_[pos][n] = adjacent( pos, n );
            Point::diagonal_[pos][n] = diagonal( pos, n );
        }
}


Point PointShiftCache::adjacent( Point pos, int n )
{
    switch ( n % 4 ) {
        case 0: return pos.L();
        case 1: return pos.T();
        case 2: return pos.R();
        case 3: return pos.B();
        default: return -1;
    }
}

Point PointShiftCache::diagonal( Point pos, int n )
{
    return adjacent( adjacent( pos, n ), n+1 );
}


int Point::hdist() const
{ 
	int c = col();
	if ( c <= 9 ) return c;
	else return 18 - c;
    //return min( col(), 18 - col() ); 
}

int Point::vdist() const
{
	int r = row();
	if ( r <= 9 ) return r;
	else return 18 - r;
    //return min( row(), 18 - row() );
}

void PointArray::remove_duplicates()
{
    std::sort( begin(), end(), PointArray::greater );
    erase( std::unique( begin(), end(), PointArray::equal ), end() );
}

// static
bool PointArray::greater( const Point& p1, const Point& p2 )
{
    return p1 > p2;
}


// static
bool PointArray::equal( const Point& p1, const Point& p2 )
{
    return (int)p1 == (int)p2;
}


char StoneBoard::get_symbol( int p, void* refData ) const
{ 
    MarkerBoard* brd = static_cast<MarkerBoard*>( refData );
    if ( brd && brd->nc_marked(p) )
    {
        if ( black(p) ) return '-';
        if ( white(p) ) return '-';
    }
    
    if ( black(p) ) return 'X';
    if ( white(p) ) return 'O';
    return '.';
}
