//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#include "StdAfx.h"
#include "Region.h"
#include "..\\Navigator\\BoardNavigator.h"


// PURPOSE: given the array of points counts its perimeter - all adjacent points,
// inlcuding those over the edge
// NOTE: Block, EmptySpace, CER and Chain count their perimeters while expanding
// MODIFIERS: static
int Region::CountPerimeter( const PointArray& points )
{
    int perimeter = 0;

    MarkerBoard temp;
    temp.nc_mark( points );

    for ( int pi = points.size(); pi-- > 0; )
    {
        Point p( points[pi] );
        if ( !p.ok() ) continue;

        for ( int n = 0; n < 4; n++ )
        {
            Point adj( p.adjacent(n) );
            if ( !adj.ok() )
            {
                perimeter++;
                continue;
            }

            if ( !temp.nc_marked(adj) )
            {
                perimeter++;
                temp.nc_mark(adj);
            }
        }
    }
    return perimeter;
}


// PURPOSE: this function counts all points, that are important to preserve 
// connectedness in the local 3x3 window around the point
// MODIFIERS: static
int Region::CountSplitPoints( const PointArray& points, PointArray* split_points )
{
    if ( points.size() < 3 ) return 0;

    PointArray sp_points;

    // first, place all points on the board
    MarkerBoard temp;
    temp.nc_mark( points );

    // now go over all points again and analyse its neibourhood
    for ( int pi = points.size(); pi-- > 0; )
    {
        Point p( points[pi] );
        if ( !p.ok() ) continue;

        // count adjacent points of the region
        int adj = 0, adj_ok = 0, n;
        for ( n = 0; n < 4; n++ )
        {
            Point pa = p.adjacent(n);
            if ( pa.ok() )
            {
                adj_ok++;
                if ( temp.nc_marked(pa) ) adj++;
            }
        }

        if ( adj <= 1 ) continue;

        // analyse shape
        if ( adj == 2 )
        {
            for ( n = 0; n < 4; n++ )
                if ( temp.marked(p.adjacent(n)) &&
                   ( temp.marked(p.adjacent(n+2)) ||
                   ( temp.marked(p.adjacent(n+1)) && !temp.marked(p.diagonal(n)) ) ) )
                {
                    sp_points.push_back(p);
                    break;
                }
        }
        else if ( adj == 3 )
        {
            for ( n = 0; n < 4; n++ )
                if ( temp.marked(p.adjacent(n)) && 
                     temp.marked(p.adjacent(n+1)) &&
                    !temp.marked(p.diagonal(n)) )
                {
                    sp_points.push_back(p);
                    break;
                }

        }
        else if ( adj == 4 )
        {
            int diag = 0;
            for ( int n = 0; n < 4; n++ )
                if ( !temp.marked(p.diagonal(n)) ) diag++;

            if ( diag >= 2 ) sp_points.push_back(p);
        }
    } // for

    if ( split_points )
        split_points->assign( sp_points.begin(), sp_points.end() );

    return sp_points.size();
}


// PURPOSE: looks for adjacent CERs and adjacent blocks of the opposite color
void StoneRegion::FindAdjacentBlocksAndCERs()
{
    MarkerArray bma( navi->allRegions.size() );
    MarkerArray cma( navi->allCERs.size() );

    for ( int ni = neibours.size(); ni-- > 0; )
    {
        Point p( neibours[ni] );
        if ( !p.ok() ) continue;

        if ( !navi->board.nc_empty(p) )
        {
            int bid = navi->nc_block_at(p)->id();
            if ( !bma.marked( bid ) )
            {
                adjOppBlocks.push_back( bid );
                bma.mark( bid );
            }
        }

        int cid = navi->cer_at(p, color())->id();
        if ( !cma.marked( cid ) )
        {
            adjCERs.push_back( cid );
            cma.mark( cid );
        }
    }
}

