//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#ifndef REGION_H_INCLUDED_
#define REGION_H_INCLUDED_


// this is base class for any regions on the board
class Region
{
public:

    Region( int id, int color, BoardNavigator* bnav ) 
     : regID_(id), color_(color), perimeter_(0), navi(bnav) {}

    virtual ~Region() {}

    // common properties accessors
    virtual int id()        const { return regID_; }
    virtual int color()     const { return color_; }
    virtual int perimeter() const { return perimeter_; }

    virtual void preprocess() = 0;
    virtual void update_props() = 0;

    // counts all points adjacent to the given region, including those ove the edge
    static int CountPerimeter( const PointArray& points );

    // counts those points of the region, which are important to preserve region
    // connection in local 3x3 window of the point; this is very important property
    // of the eye regions, because it affects static eyespace evaluation
    static int CountSplitPoints( const PointArray& points, PointArray* split_points = NULL );
    
    PointArray points;      // all points, that this region occupy
    PointArray neibours;    // all adjacent points on the board

    BoardNavigator* navi;   // pointer to the main operational class
    
protected:

    int regID_;             // identifier of the region
    int color_;             // color characteristic of the region
    int perimeter_;         // perimeter of the points occupied by the region
};


// base class for regions, that consist only of the stones
class StoneRegion: public Region
{
public:
    StoneRegion( int id, int color, BoardNavigator* bnav )
     : Region( id, color, bnav ) {}
    
    virtual ~StoneRegion() {}

    virtual void preprocess() = 0;
    virtual void update_props() = 0;

    IndexArray adjCERs;         // all adjacent CERs index
    IndexArray adjOppBlocks;    // all adjacent opponent blocks index

protected:

    void FindAdjacentBlocksAndCERs();      // looks for adjacent CERs and opponent blocks
};


// storage type for objects, derived from Region
typedef std::vector<Region*> RegionArray;


#endif // REGION_H_INCLUDED_