`//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#include "StdAfx.h"
#include "BlockFeatures.h"
#include "BoardNavigator.h"
#include "..\\Utils\\NetConfig.h"


//static
void BFeatures::usePreset_( NetConfig &netConfig, const int S1i[], const int SNi[] )
{
    std::vector<int> ts1i;
    for ( int const *ptr = S1i; *ptr != features_num; ptr++ )
        ts1i.push_back( *ptr );

    std::vector<int> tsNi;
    tsNi.assign( ts1i.begin(), ts1i.end() );
    for ( int const *ptr = SNi; *ptr != features_num; ptr++ )
        tsNi.push_back( *ptr );
    
    netConfig.setNetInputPreset( ts1i, tsNi );
}


//static
void BFeatures::usePresetA( NetConfig &netConfig )
{
    static const int S1i[] = {
        block_size,
        block_perimeter,
        block_liberties_1,
        block_liberties_2,
        block_liberties_3,
        block_chain_connections,
        block_local_majority,
        block_opp_blocks_num,
        
        block_prot_liberties,
        fri_s1_size,
        fri_s1_liberties,
        fri_s1_shared_liberties,
        
        terr_direct_liberties,
        pacer_average_power,
        pacer_sum_acc_size,
        pacer_sum_acc_perimeter,
        chain_size,
        chain_liberties,
        chain_perimeter,
        chain_adj_cer_num,
        chain_estimated_adj_terr,
        chain_ext_opp_liberties,
        chain_split_liberties,
        chain_eye_num,
        chain_eye_sum_size,
        chain_eye_sum_perimeter,
        chain_eye_sum_split_points,
        chain_pacer5_num,
        chain_pacer5_ave_power,
        chain_pacer9_num,
        chain_pacer9_ave_power,
        opp_cer_size,
        opp_cer_perimeter,
        features_num // must be the last line
    };

    static const int SNi[] = {
        //block_predicted_status,
        //chain_ave_predicted_status,
        opp_w1_predicted_status,
        opp_w2_predicted_status,
        opp_w3_predicted_status,
        opp_cw1_ave_predicted_status,
        fri_s1_predicted_status,
        block_predicted_status,
        chain_ave_predicted_status,
        features_num // must be the last line
    };

    usePreset_( netConfig, S1i, SNi );
}

//static
void BFeatures::usePresetB( NetConfig &netConfig )
{
    static const int S1i[] = {
        block_size,
        block_perimeter,
        block_liberties_1,
        block_liberties_2,
        block_liberties_3,
        block_chain_connections,
        block_local_majority,
        block_opp_blocks_num,
        block_cm_x,
        block_cm_y,
        terr_direct_liberties,
        eye_num,
        eye_sum_size,
        eye_sum_perimeter,
        eye_sum_split_points_num,
        pacer_average_power,
        pacer_sum_acc_size,
        pacer_sum_acc_perimeter,
        pacer_sum_int_size,
        pacer_sum_int_perimeter,
        pacer_sum_int_split_points,
        chain_size,
        chain_liberties,
        chain_perimeter,
        chain_adj_cer_num,
        chain_estimated_adj_terr,
        chain_ext_opp_liberties,
        chain_split_liberties,
        chain_eye_num,
        chain_eye_sum_size,
        chain_eye_sum_perimeter,
        chain_eye_sum_split_points,
        chain_pacer5_num,
        chain_pacer5_ave_power,
        chain_pacer9_num,
        chain_pacer9_ave_power,
        opp_cer_size,
        opp_cer_perimeter,
        opp_w3_size,
        opp_w3_perimeter,
        opp_w3_liberties,
        opp_w3_shared_liberties,
        opp_w3_chain_connections,
        features_num // must be the last line
    };

    static const int SNi[] = {
        block_predicted_status,
        chain_ave_predicted_status,
        opp_w3_predicted_status,
        features_num // must be the last line
    };

    usePreset_( netConfig, S1i, SNi );
}

//static
// full preset
void BFeatures::usePresetC( NetConfig &netConfig )
{
    static const int S1i[] = {
        block_size,
        block_perimeter,
        block_liberties_1,
        block_liberties_2,
        block_liberties_3,
        block_prot_liberties,
        block_chain_connections,
        block_opp_blocks_num,
        block_local_majority,
        block_cm_x,
        block_cm_y,
        block_bb_square,
        eye_num,
        eye_sum_size,
        eye_sum_perimeter,
        eye_sum_split_points_num,
        pacer_average_power,
        pacer_sum_acc_size,
        pacer_sum_acc_perimeter,
        pacer_sum_int_size,
        pacer_sum_int_perimeter,
        pacer_sum_int_split_points,
        opp_cer_size,
        opp_cer_perimeter,
        chain_blocks_num,
        chain_size,
        chain_liberties,
        chain_perimeter,
        chain_adj_cer_num,
        chain_eye_num,
        chain_eye_sum_size,
        chain_eye_sum_perimeter,
        chain_eye_sum_split_points,
        chain_ext_opp_liberties,
        chain_split_liberties,
        chain_estimated_adj_terr,
        chain_pacer5_num,
        chain_pacer5_ave_power,
        chain_pacer9_num,
        chain_pacer9_ave_power,
        opp_w1_size,
        opp_w1_perimeter,
        opp_w1_liberties,
        opp_w1_shared_liberties,
        opp_w1_atari_liberties,
        opp_w1_chain_connections,
        opp_w1_eye_num,
        opp_w1_eye_sum_size,
        opp_w1_eye_sum_perimeter,
        opp_w1_eye_sum_split_points,
        opp_cw1_size,
        opp_cw1_perimeter,
        opp_cw1_eye_num,
        opp_cw1_eye_sum_size,
        opp_cw1_eye_sum_perimeter,
        opp_cw1_eye_sum_split_points,
        opp_cw1_split_liberties,
        opp_w2_size,
        opp_w2_perimeter,
        opp_w2_liberties,
        opp_w2_shared_liberties,
        opp_w2_atari_liberties,
        opp_w2_chain_connections,
        opp_w2_eye_num,
        opp_w2_eye_sum_size,
        opp_w2_eye_sum_perimeter,
        opp_w2_eye_sum_split_points,
        opp_w3_size,
        opp_w3_perimeter,
        opp_w3_liberties,
        opp_w3_shared_liberties,
        opp_w3_chain_connections,
        opp_w3_eye_num,
        opp_w3_eye_sum_size,
        opp_w3_eye_sum_perimeter,
        opp_w3_eye_sum_split_points,
        fri_s1_size,
        fri_s1_perimeter,
        fri_s1_liberties,
        fri_s1_shared_liberties,
        fri_s1_prot_liberties,
        fri_s1_chain_connections,
        fri_s1_eye_num,
        fri_s1_eye_sum_size,
        fri_s1_eye_sum_perimeter,
        fri_s1_eye_sum_split_points,
        terr_direct_liberties,
        terr_all_friend_liberties,
        terr_all_opp_liberties,
        features_num // must be the last line
    };

    static const int SNi[] = {
        block_predicted_status,
        chain_ave_predicted_status,
        opp_w1_predicted_status,
        opp_w2_predicted_status,
        opp_w3_predicted_status,
        opp_cw1_ave_predicted_status,
        fri_s1_predicted_status,
        features_num // must be the last line
    };

    usePreset_( netConfig, S1i, SNi );
}




void BFeatures::usePresetD( NetConfig &netConfig )
{
    static const int S1i[] = {
        block_size,
        //block_cm_x,
        //block_cm_y,
        block_perimeter,
        block_liberties_1,
        block_liberties_2,
        block_liberties_3,
        block_chain_connections,
        block_local_majority,
        block_opp_blocks_num,
        terr_direct_liberties,
        pacer_average_power,
        pacer_sum_acc_size,
        pacer_sum_acc_perimeter,
        opp_cer_size,
        opp_cer_perimeter,
        chain_size,
        chain_liberties,
        chain_perimeter,
        //chain_adj_cer_num,
        chain_estimated_adj_terr,
        //chain_ext_opp_liberties,
        //chain_split_liberties,
        chain_eye_num,
        chain_eye_sum_size,
        chain_eye_sum_perimeter,
        chain_eye_sum_split_points,
        chain_pacer5_num,
        chain_pacer5_ave_power,
        chain_pacer9_num,
        chain_pacer9_ave_power,
        features_num // must be the last line
    };

    static const int SNi[] = {
        features_num // must be the last line
    };

    usePreset_( netConfig, S1i, SNi );
}


void BFeatures::fill( const Block& block, int iteration )
{
    assert( block.color() != eEmpty );
    memset( fvector, 0, features_num * sizeof( double ) );

    Chain* c0 = block.navi->allChains[ block.chainID ];
    Block* w1 = block.navi->get_block( block.opp_w1 );
    Chain* c1 = ( w1 ? block.navi->allChains[ w1->chainID ] : 0 ); 
    Block* w2 = block.navi->get_block( block.opp_w2 );
    Block* w3 = block.navi->get_block( c0->opp_w3 );
    Block* s1 = block.navi->get_block( block.fri_s1 );

    const std::vector<int> *preset = ( iteration == 0 ? &netConfig_.npVec1i_ : &netConfig_.npVecNi_ );
    for ( int i = 0, imax = preset->size(); i < imax; i++ )
    {
        switch ( preset->at(i) )
        {
            case block_size:                    fvector[i] = block.points.size(); break;
            case block_perimeter:               fvector[i] = block.perimeter(); break;
            case block_liberties_1:             fvector[i] = block.liberties[0].size(); break;
            case block_liberties_2:             fvector[i] = block.liberties[1].size(); break;
            case block_liberties_3:             fvector[i] = block.liberties[2].size(); break;
            case block_prot_liberties:          fvector[i] = block.protected_liberties; break;
            case block_chain_connections:       fvector[i] = block.connections.size(); break;
            case block_opp_blocks_num:          fvector[i] = block.adjOppBlocks.size(); break;
            case block_local_majority:          fvector[i] = block.local_majority; break;
            case block_cm_x:                    fvector[i] = block.cm_x; break;
            case block_cm_y:                    fvector[i] = block.cm_y; break;
            case block_bb_square:               fvector[i] = block.BBSquare(); break;
            case eye_num:                       fvector[i] = block.eye_num; break;
            case eye_sum_size:                  fvector[i] = block.eye_sum_size;break;
            case eye_sum_perimeter:             fvector[i] = block.eye_sum_perimeter; break;
            case eye_sum_split_points_num:      fvector[i] = block.eye_sum_split_points; break;
            case pacer_average_power:           fvector[i] = block.pacer_average_power; break;
            case pacer_sum_acc_size:            fvector[i] = block.pacer_sum_acc_size;break;
            case pacer_sum_acc_perimeter:       fvector[i] = block.pacer_sum_acc_perimeter; break;
            case pacer_sum_int_size:            fvector[i] = block.pacer_sum_int_size;break;
            case pacer_sum_int_perimeter:       fvector[i] = block.pacer_sum_int_perimeter; break;
            case pacer_sum_int_split_points:    fvector[i] = block.pacer_sum_int_split_points; break;
            case opp_cer_size:                  fvector[i] = block.opp_cer_size;break;
            case opp_cer_perimeter:             fvector[i] = block.opp_cer_perimeter; break;
            case terr_direct_liberties:         fvector[i] = block.terr_direct_liberties; break;
            case terr_all_friend_liberties:     fvector[i] = block.terr_all_friend_liberties; break;
            case terr_all_opp_liberties:        fvector[i] = block.terr_all_opp_liberties; break;
            case block_predicted_status:        fvector[i] = block.predicted_status; break;

            case chain_blocks_num:              fvector[i] = c0->blocks.size(); break;
            case chain_size:                    fvector[i] = c0->points.size(); break;
            case chain_liberties:               fvector[i] = c0->liberties.size(); break;
            case chain_perimeter:               fvector[i] = c0->perimeter(); break;
            case chain_adj_cer_num:             fvector[i] = c0->adjCERs.size(); break;
            case chain_eye_num:                 fvector[i] = c0->eye_num; break;
            case chain_eye_sum_size:            fvector[i] = c0->eye_sum_size;break;
            case chain_eye_sum_perimeter:       fvector[i] = c0->eye_sum_perimeter; break;
            case chain_eye_sum_split_points:    fvector[i] = c0->eye_sum_split_points; break;
            case chain_ext_opp_liberties:       fvector[i] = c0->extOppLiberties.size(); break;
            case chain_split_liberties:         fvector[i] = c0->split_liberties.size(); break;
            case chain_estimated_adj_terr:      fvector[i] = c0->territory.size(); break;
            case chain_pacer5_num:              fvector[i] = c0->pacer5_num; break;
            case chain_pacer5_ave_power:        fvector[i] = c0->pacer5_avepow; break;
            case chain_pacer9_num:              fvector[i] = c0->pacer9_num; break;
            case chain_pacer9_ave_power:        fvector[i] = c0->pacer9_avepow; break;
            case chain_ave_predicted_status:    fvector[i] = c0->average_status; break;

            case opp_w1_size:                   if ( w1 ) fvector[i] = w1->points.size(); break;
            case opp_w1_perimeter:              if ( w1 ) fvector[i] = w1->perimeter(); break;
            case opp_w1_liberties:              if ( w1 ) fvector[i] = w1->liberties[0].size(); break;
            case opp_w1_shared_liberties:       if ( w1 ) fvector[i] = w1->SharedLiberties( block ); break;
            case opp_w1_atari_liberties:        if ( w1 ) fvector[i] = w1->autoatari_liberties; break;
            case opp_w1_chain_connections:      if ( w1 ) fvector[i] = w1->connections.size(); break;
            case opp_w1_eye_num:                if ( w1 ) fvector[i] = w1->eye_num; break;
            case opp_w1_eye_sum_size:           if ( w1 ) fvector[i] = w1->eye_sum_size; break;
            case opp_w1_eye_sum_perimeter:      if ( w1 ) fvector[i] = w1->eye_sum_perimeter; break;
            case opp_w1_eye_sum_split_points:   if ( w1 ) fvector[i] = w1->eye_sum_split_points; break;
            case opp_w1_predicted_status:       if ( w1 ) fvector[i] = w1->predicted_status; break;

            case opp_cw1_size:                  if ( c1 ) fvector[i] = c1->points.size(); break;
            case opp_cw1_perimeter:             if ( c1 ) fvector[i] = c1->perimeter(); break;
            case opp_cw1_eye_num:               if ( c1 ) fvector[i] = c1->eye_num; break;
            case opp_cw1_eye_sum_size:          if ( c1 ) fvector[i] = c1->eye_sum_size; break;
            case opp_cw1_eye_sum_perimeter:     if ( c1 ) fvector[i] = c1->eye_sum_perimeter; break;
            case opp_cw1_eye_sum_split_points:  if ( c1 ) fvector[i] = c1->eye_sum_split_points; break;
            case opp_cw1_split_liberties:       if ( c1 ) fvector[i] = c1->split_liberties.size(); break;
            case opp_cw1_ave_predicted_status:  if ( c1 ) fvector[i] = c1->average_status; break;

            case opp_w2_size:                   if ( w2 ) fvector[i] = w2->points.size(); break;
            case opp_w2_perimeter:              if ( w2 ) fvector[i] = w2->perimeter(); break;
            case opp_w2_liberties:              if ( w2 ) fvector[i] = w2->liberties[0].size(); break;
            case opp_w2_shared_liberties:       if ( w2 ) fvector[i] = w2->SharedLiberties( block ); break;
            case opp_w2_atari_liberties:        if ( w2 ) fvector[i] = w2->autoatari_liberties; break;
            case opp_w2_chain_connections:      if ( w2 ) fvector[i] = w2->connections.size(); break;
            case opp_w2_eye_num:                if ( w2 ) fvector[i] = w2->eye_num; break;
            case opp_w2_eye_sum_size:           if ( w2 ) fvector[i] = w2->eye_sum_size; break;
            case opp_w2_eye_sum_perimeter:      if ( w2 ) fvector[i] = w2->eye_sum_perimeter; break;
            case opp_w2_eye_sum_split_points:   if ( w2 ) fvector[i] = w2->eye_sum_split_points; break;
            case opp_w2_predicted_status:       if ( w2 ) fvector[i] = w2->predicted_status; break;

            case opp_w3_size:                   if ( w3 ) fvector[i] = w3->points.size(); break;
            case opp_w3_perimeter:              if ( w3 ) fvector[i] = w3->perimeter(); break;
            case opp_w3_liberties:              if ( w3 ) fvector[i] = w3->liberties[0].size(); break;
            case opp_w3_shared_liberties:       if ( w3 ) fvector[i] = w3->SharedLiberties( block ); break;
            case opp_w3_chain_connections:      if ( w3 ) fvector[i] = w3->connections.size();
            case opp_w3_eye_num:                if ( w3 ) fvector[i] = w3->eye_num; break;
            case opp_w3_eye_sum_size:           if ( w3 ) fvector[i] = w3->eye_sum_size; break;
            case opp_w3_eye_sum_perimeter:      if ( w3 ) fvector[i] = w3->eye_sum_perimeter; break;
            case opp_w3_eye_sum_split_points:   if ( w3 ) fvector[i] = w3->eye_sum_split_points; break;
            case opp_w3_predicted_status:       if ( w3 ) fvector[i] = w3->predicted_status; break;

            case fri_s1_size:                   if ( s1 ) fvector[i] = s1->points.size(); break;
            case fri_s1_perimeter:              if ( s1 ) fvector[i] = s1->perimeter(); break;
            case fri_s1_liberties:              if ( s1 ) fvector[i] = s1->liberties[0].size(); break;
            case fri_s1_shared_liberties:       if ( s1 ) fvector[i] = s1->SharedLiberties( block ); break;
            case fri_s1_prot_liberties:         if ( s1 ) fvector[i] = s1->protected_liberties; break;
            case fri_s1_chain_connections:      if ( s1 ) fvector[i] = s1->connections.size(); break;
            case fri_s1_eye_num:                if ( s1 ) fvector[i] = s1->eye_num; break;
            case fri_s1_eye_sum_size:           if ( s1 ) fvector[i] = s1->eye_sum_size; break;
            case fri_s1_eye_sum_perimeter:      if ( s1 ) fvector[i] = s1->eye_sum_perimeter; break;
            case fri_s1_eye_sum_split_points:   if ( s1 ) fvector[i] = s1->eye_sum_split_points; break;
            case fri_s1_predicted_status:       if ( s1 ) fvector[i] = s1->predicted_status; break;
        }
    }
    
    expert_estimate = block.expert_estimate;
    blockId = block.id();
    boardId = block.navi->boardId;
    x = block.points[0].col();
    y = block.points[0].row();
    
    //if ( expert_estimate == 1.0 )
    //    expert_estimate = 0.8;
    //else
    //    expert_estimate = 0.2;
}
