//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#ifndef BOARD_NAVIGATOR_H_INCLUDED_
#define BOARD_NAVIGATOR_H_INCLUDED_

#include "Boards.h"
#include "..\\Utils\\NetConfig.h"

class BFeatures;

// let's make clear how expert works:
// 1. when reading board, if there are marked stones, they won't be placed
// on the expert board, so that when the EndPosition is divided into
// blocks of stones, block will verify it's stones on expert board...
// and if block doesn't see one of it's stones on expert board, then he
// makes his expert_estimate status DEAD... now
// 2. when board navigator will perform new step, it'll check, whether
// there is expert board in the system, and if it is there, navigator will
// trigger neural network learning and give it block's expert_estimate
class BoardNavigator
{
public:

    static int ScoreEndOfGame( const StoneBoard& EndPosition, const NetFactory* nf );

    const MarkerBoard* expert;  // expert block life & death classification
    bool isTestingBoard_;
    
    StoneBoardMasked board;     // main board with stones
    PowerBoard  power;          // influence board

    PowerBoard  zInfluence;     // Zobrist influence estimation
    PowerBoard  zTerritory;     // Zobrist territory estimation
    //IndexBoard  lgip;           // LGroup index plane
    IndexBoard  blip;           // all blocks index plane
    IndexArray  sbIndex;        // index of all blocks of stones
    IndexArray  esIndex;        // index of all empty spaces (block of empty points)
    IndexBoard  berip;          // black-enclosed regions index plane
    IndexBoard  werip;          // white-enclosed regions index plane
    IndexBoard  cip;

    RegionArray allRegions;     // all colored regions storage (black, white, empty)
    ChainArray  allChains;      // all chains storage
    CERArray    allCERs;        // color-enclosed regions object storage
    //LGroupArray allGroups;

    IndexArray biqIndex;        // all blocks-in-question index
    int netError;
    int boardId;

protected:

    friend class TrainingFactory;
    friend class MorphEngine;
	const NetFactory* netFactory_;

    BoardNavigator( const StoneBoard& EndPosition, const NetFactory* nf, int brdId = 0 );
   ~BoardNavigator();

    void Preprocess();
    void ProcessBoard( const NetConfig &netConfig );
    int  CountTerritory();

    void CalcRoughInfluence();
    void CheckTerritoryAndSelectBIQs();
    void PrepareGradingStep();
    void PerformGradingStep( int iteration, const NetConfig &netConfig );
    
    void PreprocessAndStoreFeatures( std::vector< BFeatures* > &features, const NetConfig &netConfig );

public:

    ESpace* espace_at( const Point& pos );
    Block*  block_at( const Point& pos );
    Chain*  chain_at( const Point& pos );
    CER*    cer_at  ( const Point& pos, int color );

    ESpace* get_espace( unsigned int index );
    Block*  get_block( unsigned int index );
    Chain*  get_chain( unsigned int index );
    CER*    get_cer  ( unsigned int index );

    Block*  nc_block_at( const Point& pos ) { return static_cast<Block*>( allRegions[blip.get(pos)] ); }
    Block*  nc_get_block( int index ) { return static_cast<Block*>( allRegions[index] ); }
    ESpace* nc_get_espace( int index ) { return static_cast<ESpace*>( allRegions[index] ); }
    void    print();
};


class TrainingFactory
{
private:
    std::vector<StoneBoard*> positions_;
    std::vector<MarkerBoard*> experts_;
    std::vector<int> errors_;
    //int sumError_;

	const NetFactory* netFactory_;
	const NetConfig* netConfig_;
	
public:
	
	TrainingFactory( const NetFactory* factory, const NetConfig* config ) 
		: netFactory_(factory), netConfig_(config) {}
   ~TrainingFactory();

    void AddFinalPosition( const StoneBoard& EndPosition, 
						   const MarkerBoard& ExpertBoard );
    void ResetErrors();
    
    void Process();
	//int GetSumError() const { return sumError_; }

    void ParsePositionAndStoreFeatures(
        const StoneBoard& EndPosition, 
        const MarkerBoard& ExpertBoard,
        int boardId );
    void ProcessParsed( bool testIt );
    std::vector< BFeatures* > features_;
};


#endif // BOARD_NAVIGATOR_H_INCLUDED_