//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************

#ifndef RPROPLAYER_H_INCLUDED_
#define RPROPLAYER_H_INCLUDED_

#include "RPropNeuron.h"

class TheLayer
{
public:
    int     inputSize_;  // input size
    int     layerSize_;
    double *inputPtr_;   // ptr to input vector
    double *output_;     // output vector
    double *nDelta_;
    double *sum_ni_;

    std::vector<TheNeuron*> neurons_;

    // ctr
    TheLayer() : inputPtr_(0), output_(0), nDelta_(0) {}
    
    // dtr
    ~TheLayer()
    {
        if ( output_ ) { delete [] output_; output_ = 0; }
        if ( nDelta_ ) { delete [] nDelta_; nDelta_ = 0; }
        if ( sum_ni_ ) { delete [] sum_ni_; sum_ni_ = 0; }
        for ( unsigned int i = 0; i < neurons_.size(); i++ )
            delete neurons_[i];
        neurons_.clear();
    }

    void allocate( int layerSize, int inputSize, double *inputPtr )
    {
        inputSize_ = inputSize;
        layerSize_ = layerSize;
        inputPtr_ = inputPtr;

        output_ = new double[ layerSize ];
        nDelta_ = new double[ layerSize ]; memset( nDelta_, 0, sizeof( double ) * layerSize_ );
        sum_ni_ = new double[ layerSize ];
        
        for ( int i = 0; i < layerSize_; i++ )
            neurons_.push_back( new TheNeuron( inputSize_, inputPtr_, output_[i], nDelta_[i] ) );
    }

    void apply()
    {
        for ( int i = 0; i < layerSize_; i++ )
            neurons_[i]->apply();
    }
};


#endif // RPROPLAYER_H_INCLUDED_
