//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#include "StdAfx.h"
#include "RPropNet.h"

NetFactory::~NetFactory()
{
    for ( unsigned int n = 0; n < networks.size(); n++ )
        delete networks[n];

    networks.clear();
}


int NetFactory::CreateNetwork( const std::vector<int>& topology, const std::string& filename, RPropNet::NetType type )
{
    //printf( "creating network <%s>\n", filename.c_str() );
    networks.push_back( new RPropNet( topology, filename, type ) );
    return networks.size() - 1;
}

void NetFactory::StartBatchTraining()
{
    //if ( netConfig_.getTrainOnlyOutput() )
    //    networks.back()->startBatchTraining();
    //else for ( int n = 0; n < networks.size(); n++ )
    //    networks[n]->startBatchTraining();

    // we train only output network
    networks.back()->startBatchTraining();
    printf( "\n%.3d. ", networks.back()->netEpochCount() + 1 );
}

void NetFactory::FinishBatchTraining()
{
    for ( int n = 0; n < networks.size(); n++ )
    {
        printf( "NN%.1d: training set error = %6.5g%%, testing set error = %6.5g%%\n",
            n+1, networks[n]->netTrainError() * 100, networks[n]->netTestError() * 100 );
        //int ln = networks[n]->layersNum_;
        //TheLayer &la = networks[n]->layers_[ ln - 1 ];
        //printf( "---- w1 = %6.5g, w2 = %6.5g\n", la.neurons_[0]->weights_[0], la.neurons_[0]->weights_[1] );
        networks[n]->finishBatchTraining();
    }
}


int NetFactory::NetsNumber() const
{
    return networks.size();
}


RPropNet* NetFactory::Network( unsigned int index ) const
{
    if ( index >= 0 && index < networks.size() ) return networks[index];
    return NULL;
}


