//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#include "StdAfx.h"
#include "IOSgf.h"
#include "IOLog.h"
#include "..\\Navigator\\Boards.h"

bool LoadFile( const char* file_name, char* buffer )
{
    assert( file_name && buffer );

    FILE* sgf_file = fopen( file_name, "rb" );
    if ( !sgf_file )
    {
        //char error_buf[256];
        //sprintf( error_buf, "SGF IO error: file %s not found\n", file_name );
        //LOG::Instance().Write( error_buf );
		LogError() << "SGF IO error: file " << file_name << " not found\n";
        return false;
    }

    char ch;
    int sgf_size = 0;
    while ( !feof(sgf_file) )
    {
        fread( &ch, 1, 1, sgf_file );
        if ( ch != 32 && ch != 9 && ch != 10 && ch != 13 )
            buffer[sgf_size++] = ch;
    }
    buffer[ sgf_size ] = 0;
    fclose( sgf_file );

    if ( sgf_size == 0 )
    {
        //LOG::Instance().Write( "SGF IO error: file have zero size\n" );
		LogError() << "SGF IO error: file have zero size\n";
        return false;
    }

    return true;
}


double GetGameResult( char* buffer )
{
    assert( buffer && strlen(buffer) );

    char *ss, *ps, str[100];
    double GameResult = 400, M = 0;
    if ( (ss = strstr(buffer, "RE[")) && (ps = strstr(ss, "]")) && (ps-ss < 100) && (ps-ss > 5) )
    {
        GameResult = 0;
        strncpy(str, ss+3, ps-ss-3); str[ps-ss-3] = 0;

        if ( str[0] == 'W' || str[0] == 'w' ) M = -1;
        else
            if ( str[0] == 'B' || str[0] == 'b' ) M =  1;
        else GameResult = 400;

        if ( str[1] != '+' || str[2] == 'r' || str[2] == 'R' 
             || str[2] == 't' || str[2] == 'T' ) GameResult = 400;
        strncpy(str, ss+5, ps-ss-5); str[ps-ss-5] = 0;

        if ( GameResult != 400 )
            GameResult = atof(str) * M;
    }

    return GameResult;
}


bool LoadSGF( const char* fileName, StoneBoard* pEndPosition, MarkerBoard* ExpertBoard,
              int* pBlackDead, int* pWhiteDead, double* pKomi, double* pResult )
{
    if ( !pEndPosition )
        return false;

    static char buffer[32000];
    char *ss, *ps, str[100];

    if ( !LoadFile( fileName, buffer ) ) return false;

    // read board size
    int BoardSize = 0;
    if ( (ss = strstr(buffer, "SZ[")) && (ps = strstr(ss, "]")) && (ps-ss < 100) )
    {
        strncpy(str, ss+3, ps-ss-3); str[ps-ss-3] = 0; BoardSize = atoi(str);
        if ( BoardSize != 19 )
        {
            //LOG::Instance().Write( "SGF IO error: wrong board size\n" );
			LogError() << "SGF IO error: wrong board size\n";
            return false;
        }
    }

    // read game komi
    double Komi = -1;
    if ( (ss = strstr(buffer, "KM[")) && (ps = strstr(ss, "]")) && (ps-ss < 100) )
    {
        strncpy(str, ss+3, ps-ss-3); str[ps-ss-3] = 0; Komi = atof(str);
    }

    if ( Komi < 0 )
		LogWarning() << "komi not specified\n";

    double GameResult = GetGameResult( buffer );
    if ( GameResult == 400 )
		LogWarning() << "wrong result format or value\n";

    // read game moves and expert marks
    PointArray moves, handi, colors;
    unsigned int n, nmax = strlen(buffer) - 5;
    for ( n = 1; n < nmax; n++ )
    {
        if ( ((buffer[n] == 'B' && buffer[n-1] == ';') || 
              (buffer[n] == 'W' && buffer[n-1] == ';')) && buffer[n+1] == '[' )
        {
            if ( buffer[n+2] == ']' )
            {
                moves.push_back(-1);    // pass
                colors.push_back( eEmpty );
                n += 2;
            }
            else
            {
                if ( buffer[n+4] != ']' )
                {
					LogError() << "wrong move format\n";
                    return false;
                }

                Point p( buffer[n+2]-'a', buffer[n+3]-'a' );
                if ( !p.ok() && p != 380 )
                {
                    LogError() << "wrong move position\n";
                    return false;
                }

                if ( p == 380 ) // pass
                {
                    moves.push_back( -1 );
                    colors.push_back( eEmpty );
                }
                else
                {
                    moves.push_back( p );
                    if ( buffer[n] == 'B' ) colors.push_back( eBlack );
                    else
                        if ( buffer[n] == 'W' ) colors.push_back( eWhite );
                }
                n += 4;
            }
        }

        if ( (buffer[n] == 'B' || buffer[n] == 'W') && buffer[n-1] == 'T' &&
             buffer[n+1] == '[' && buffer[n+4] == ']' )
        {
            n++;

            while ( n < strlen(buffer) - 4 )
            {
                if ( buffer[n] != '[' || buffer[n+3] != ']' ) break;

                Point p( buffer[n+1] - 'a', buffer[n+2] - 'a' );

                if ( !p.ok() )
                {
                    LogError() << "wrong mark position\n";
                    return false;
                }

                if ( ExpertBoard )
                    ExpertBoard->nc_mark(p);

                n += 4;
            }
        }

        if ( buffer[n] == 'A' && buffer[n+1] == 'B' && buffer[n+2] == '[' && buffer[n+5] == ']' )
        {
            n += 2;
            while ( n < strlen(buffer) - 4 )
            {
                if ( buffer[n] != '[' || buffer[n+3] != ']' ) break;

                Point p( buffer[n+1] - 'a', buffer[n+2] - 'a' );

                if ( !p.ok() )
                {
                    LogError() << "wrong mark position\n";
                    return false;
                }

                handi.push_back(p);
                n += 4;
            }
        }
    }

    // place all stones on the board
    int DEAD[2] = { 0, 0 };
    StoneBoard& board = *pEndPosition;

    int m, msize = moves.size();
    for ( m = handi.size() - 1; m >= 0; m-- )
        board.set( handi[m], eBlack );

    for ( m = 0; m < msize; m++ )
    {
        int color = colors[m];
        Point p( moves[m] );

        if ( p == -1 ) continue;
        board.set( p, color );

        for ( n = 0; n < 4; n++ )
        {
            Point padj = p.adjacent(n);
            if ( board.enemy(padj, color) )
            {
                MarkerBoard temp;
                PointArray group;
                int dame = 0;
                unsigned int k = 0;

                group.push_back(padj);
                temp.nc_mark(padj);

                while ( k < group.size() )
                {
                    for ( int i = 0; i < 4; i++ )
                    {
                        Point adj( group[k].adjacent(i) );

                        if ( board.empty(adj) )
                        {
                            dame++;
                            k = group.size();
                            break; 
                        }

                        if ( board.enemy(adj, color) && !temp.marked(adj) )
                        {
                            group.push_back(adj);
                            temp.nc_mark(adj);
                        }
                    }
                    k++;
                }

                if ( dame == 0 )
                {
                    for ( k = 0; k < group.size(); k++ )
                        board.set( group[k], eEmpty );
                    DEAD[2-color] += group.size();
                    group.clear();
                }
            }
        }
    }

    if ( pBlackDead )
        *pBlackDead = DEAD[0];

    if ( pWhiteDead )
        *pWhiteDead = DEAD[1];

    if ( pKomi )
        *pKomi = Komi;

    if ( pResult )
        *pResult = GameResult;

    return true;
}

