//************************************************************************
//   Copyright (C) 2004-2006 Maxim I. Guisov                            //
//                                                                      //
// Permission to use, copy, modify, distribute and sell this software   //
// and its documentation for any purpose is hereby granted without fee, //
// provided that the above copyright notice appear in all copies and    //
// that both that copyright notice and this permission notice appear    //
// in supporting documentation. Author makes no representations         //
// about the suitability of this software for any purpose.              //
// It is provided "as is" without express or implied warranty.          //
//                                                                      //
//************************************************************************
#ifndef MORPH_ENGINE_H_INCLUDED_
#define MORPH_ENGINE_H_INCLUDED_

#include "..\\Navigator\\Boards.h"

class Block;
class BoardNavigator;

class MorphEngine
{
public:

    // counts all adjacent points on the board and fills 'neibours'
    static void GetNeibours( const PointArray& points, PointArray* neibours );

    // counts first, second and third-order liberties of the given block
    static void GetLiberties( Block* block );

    // applies dilation operator four times to estimate stone influence over the board
    static void EstimateInfluence( PowerBoard* power, const StoneBoard& board );

    // uses dilation and erasion operators to estimate potential territory
    static void EstimateTerritory( PowerBoard* terri, const StoneBoard& board );

    // dilates each stone power in all directions 4 times
    static void PropagatePower( SpPowerBoard* result, BoardNavigator& navi );

    static void PropagatePower( SpPowerBoard* result, const StoneBoard& board );

private:

    // bit board operations (for block morphology)
    static void bb_set_points( ULONG bb[bSize1D], const PointArray& points );
    static void bb_get_points( ULONG const bb[bSize1D], PointArray* points );
    static void bb_clear  ( ULONG src[bSize1D] );
    static void bb_dilate ( ULONG dst[bSize1D], ULONG const src[bSize1D] );
    static void bb_copy   ( ULONG dst[bSize1D], ULONG const src[bSize1D] );
    static void bb_and_not( ULONG op1[bSize1D], ULONG const op2[bSize1D] );
    static void bb_or     ( ULONG op1[bSize1D], ULONG const op2[bSize1D] );

    // numerical morphology for the whole board
    static void board_init( PowerBoard* index, const StoneBoard& board );
    static void board_dilate_z( PowerBoard* index );
    static void board_erase_z( PowerBoard* index );
};


#endif // MORPH_ENGINE_H_INCLUDED_