// mcSEOG.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "time.h"

void ProcessFile( int index, TrainingFactory &trainingFactory_/*, NetFactory &netFactory_*/ )
{
    StoneBoard EndPosition;
    MarkerBoard ExpertBoard;
    static char buf[256];

    sprintf( buf, "_%.3d\\_%.6d.sgf", index / 1000, index );
    if ( !LoadSGF( buf, &EndPosition, &ExpertBoard, 0, 0, 0, 0 ) )
    {
        //printf( "Couldn't load an sgf file <%s>\n", buf );
        return;
    }
    
    //if ( index == 10061 )
    //{
    //    printf( "\n" );
    //    printf( "\n" );
    //    EndPosition.print( &ExpertBoard );
    //    exit(0);
    //}
        
    // networks cascade
    //if ( netFactory_.NetsNumber() > 1 )
    //    trainingFactory_.AddFinalPosition( EndPosition, ExpertBoard );
    //else
    trainingFactory_.ParsePositionAndStoreFeatures( EndPosition, ExpertBoard, index );
}


int main(int argc, char* argv[])
{
    char *suffix[] = { "RPROP", "iRPROP", "SARP10", "SARP15", "SARP20", "CONVRP" };
    RPropNet::NetType netType = RPropNet::SARP15;
    int netNum = 1;
    int firstLayerSize = 32;
    int hiddenLayersNum = 1;
    int randomSeed = time(0);
    int trainingSetSize = 1024;
    int maxEpoch = 80000;
    int batchTrainsNum = 1;
    int batchTrainsStart = 1;
    
    bool TEST_IT = false;

    NetConfig netConfig_;
    netConfig_.setStartOnHalfTS( false );
    netConfig_.setDropSolved( false );
    netConfig_.setRefreshDrop( true );
    //netConfig_.setTrainOnlyOutput( true );

    BFeatures::usePresetA( netConfig_ );

    try
    {
        if ( argc < 10 )
            throw "wrong parameters number";
        
        if ( strcmp( argv[1], "rprop" ) == 0 )
            netType = RPropNet::RPROP;
        else
        if ( strcmp( argv[1], "impro" ) == 0 )
            netType = RPropNet::iRPROP;
        else
        if ( strcmp( argv[1], "sarp10" ) == 0 )
            netType = RPropNet::SARP10;
        else
        if ( strcmp( argv[1], "sarp15" ) == 0 )
            netType = RPropNet::SARP15;
        else
        if ( strcmp( argv[1], "sarp20" ) == 0 )
            netType = RPropNet::SARP20;
        else
        if ( strcmp( argv[1], "conv" ) == 0 )
            netType = RPropNet::CONVRP;
        else
            throw "wrong network learning method";

        netNum = atoi( argv[2] );
        if ( netNum < 1 || netNum > 3 )
            throw "wrong number of networks in cascade, must be in 1..3";

        firstLayerSize = atoi( argv[3] );
        if ( firstLayerSize < 1 || firstLayerSize > 299 )
            throw "wrong first layer size, must be in 1..299";

        hiddenLayersNum = atoi( argv[4] );
        if ( hiddenLayersNum < 1 || hiddenLayersNum > 3 )
            throw "wrong number of hidden layers, must be in 1..3";

        if ( strcmp( argv[5], "A" ) == 0 )
            BFeatures::usePresetA( netConfig_ );
        else
        if ( strcmp( argv[5], "B" ) == 0 )
            BFeatures::usePresetB( netConfig_ );
        else
        if ( strcmp( argv[5], "C" ) == 0 )
            BFeatures::usePresetC( netConfig_ );
        else
            throw "wrong block properties set";

        trainingSetSize = atoi( argv[6] );
        if ( trainingSetSize < 100 )
            throw "wrong training set size, must be > 100";

        batchTrainsNum = atoi( argv[7] );
        if ( batchTrainsNum < 1 || batchTrainsNum > 100 )
            throw "wrong number of networks to train, must be in 1..100";

        batchTrainsStart = atoi( argv[8] );
        if ( batchTrainsStart < 1 || batchTrainsStart > 100 )
            throw "wrong first network to train, must be IN 1..100";

        maxEpoch = atoi( argv[9] );
        if ( maxEpoch < 1 || maxEpoch > 100000 )
            throw "wrong number of training epoch, must be in 1..100000";
    }
    catch ( const char* ex )
    {
        printf("Error: %s\n", ex);
        printf("Please use following command syntax: \n");
        printf("%s <type> <num> <L1S> <HLN> <prop> <TSS> <batch> <bstart> <epoch>\n", argv[0]);
        printf("  where: type -- learning method: rprop, impro, sarp10, sarp15, sarp20, conv\n");
        printf("         num  -- number of cascaded networks: 1..3\n");
        printf("         L1S  -- first hidden layer size: 10..299\n");
        printf("         HLN  -- number of hidden layers\n");
        printf("                 (each will be half-sized compared to prev)\n");
        printf("         prop -- set of block properties, that network works with: A, B or C\n");
        printf("         TSS  -- training set size\n");
        printf("         batch -- number of networks, trained from scratch\n");
        printf("         bstart -- first network number to train\n");
        printf("         epoch -- maximum number of training epoch\n");
        return 0;
    }

    const char *netNameSuffix = suffix[ netType ];
    srand( randomSeed );

    NetFactory netFactory_( netConfig_ );
    NNTopology topology;
    topology.push_back( netConfig_.npVec1i_.size() );
    for ( int h = 0; h < hiddenLayersNum; h++ )
    {
        topology.push_back( firstLayerSize );
        firstLayerSize /= 2;
    }
    topology.push_back(1);

    //char buf[256];
    //sprintf_s( buf, 256, "01_%s_n1_%di_%dh.rpn", 
    //    netNameSuffix, topology[0], topology[1] );
    netFactory_.CreateNetwork( topology, "temp", netType );
    
    for ( int net = 1; net < netNum; net++ )
    {
        topology[0] = netConfig_.npVecNi_.size();
        //topology[1] = 2;
        //sprintf_s( buf, 256, "01_%s_n%d_%di_%dh.rpn", 
        //    netNamePrefix, netNameSuffix, net+1, topology[0], topology[1] );
        netFactory_.CreateNetwork( topology, "temp", netType );
    }

    TrainingFactory trainingFactory_( &netFactory_, &netConfig_ );


    for ( int b = 0; b < batchTrainsNum; b++ )
    {
        for ( int n = 0; n < netFactory_.NetsNumber(); n++ )
        {
            char buf[256];
            sprintf_s( buf, 256, "%.2x_%s_n%d_%di_%dh.rpn", 
                batchTrainsStart + b, netNameSuffix, n+1, topology[0], topology[1] );
            
            netFactory_.Network(n)->loadAs( buf, netType );
        }
        
        if ( b == 0 )
        {
            // LOAD POSITIONS
            printf("loading input data [");
            double p = 0;
            int pi = 0;
            for ( int i = 0; i < trainingSetSize; i++ )
            {
                ProcessFile( i, trainingFactory_/*, netFactory_*/ );
                if ( ( p += 50.0 / trainingSetSize ) > pi )
                {
                    printf(".");
                    pi++;
                }
            }
            printf("]\n\n");
        }
        
        double deltaTestError = 100;
        double prevTestError = 100;
        while ( netFactory_.Last()->netEpochCount() < maxEpoch )
        {
            if ( !TEST_IT )
                netFactory_.StartBatchTraining();
            
            //if ( netFactory_.NetsNumber() > 1 )
            //    trainingFactory_.Process();
            //else 
            trainingFactory_.ProcessParsed( TEST_IT );
                
            //double netTestError = netFactory_.Last()->netTestError();
            netFactory_.FinishBatchTraining();
            
            //if ( deltaTestError * fabs( netTestError - prevTestError ) < 1.0e-16   )
            //{
            //    printf("network converged!\n");
            //    break;
            //}
            //deltaTestError = fabs( netTestError - prevTestError );
            //prevTestError = netTestError;
            
            if ( netFactory_.Last()->netConverged() )
            {
                printf("network converged!\n");
                break;
            }
        }
    }    
    return 0;
}

