// sortExcelData.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "math.h"

static const char* suffix[] = { "IRPROP", "RPROP", "SARP10", "SARP15", "SARP20" };


void process_diag_conv( std::string &str )
{
    FILE *file = 0;
    double report[5][4];
    int epoch[5];

    memset( report, 0, sizeof( double ) * 5 * 4 );
    memset( epoch, 0, sizeof( int ) * 5 );

    int imax = 10;
    char buf[256];

    for ( int fi = 0; fi < 5; fi++ )
    {
        float me1 = 100, me2 = 100;
        for ( int iter = 1; iter <= imax; iter++ )
        {
            sprintf_s( buf, 256, "%s_%s_%.2x.out", str.c_str(), suffix[fi], iter );
            printf( "%s\n", buf );
            fopen_s( &file, buf, "rt" );
            if ( ! file )
            {
                printf( "can't find file <%s>\n", buf );
                exit(1);
            }

            float e1 = 0, e2 = 0;
            //while ( !feof( file ) )
            while ( fscanf_s( file, "%f, %f\n", &e1, &e2 ) != EOF )
            {
                if ( e1 < me1 ) me1 = e1;
                if ( e2 < me2 ) me2 = e2;
                epoch[fi] ++;
            }

            report[fi][0] += sqrt(e1 / 100.0);
            report[fi][1] += sqrt(e2 / 100.0);

            fclose( file );
            printf( "\n\n" );
        }
        report[fi][2] = sqrt(me1 / 100.0);
        report[fi][3] = sqrt(me2 / 100.0);
    }

    sprintf_s( buf, 256, "%s_conv.txt", str.c_str() );
    fopen_s( &file, buf, "wt" );
    for ( int fi = 0; fi < 5; fi++ )
    {
        fprintf_s( file, "%7.5g, %7.5g, %d, %7.5g, %7.5g\n", 
            report[fi][0] / imax, 
            report[fi][1] / imax,
            epoch[fi] / imax, report[fi][2], report[fi][3] );
    }

    fclose( file );
}


void process_diag_mdiff( std::string &str )
{
    FILE *file = 0;

    const int eplim = 2000;
    double report[5][eplim][2];
    memset( report, 0, sizeof( double ) * 5 * eplim * 2 );

    char buf[256];
    int imax = 10;
    for ( int iter = 1; iter <= imax; iter++ )
    {
        for ( int fi = 0; fi < 5; fi++ )
        {
            sprintf_s( buf, 256, "%s_%s_%.2x.out", str.c_str(), suffix[fi], iter );
            printf( "%s\n", buf );
            fopen_s( &file, buf, "rt" );
            if ( ! file )
            {
                printf( "can't find file <%s>\n", buf );
                exit(1);
            }

            float e1, e2;
            float me1 = 100, me2 = 100;
            for ( int ep = 0; ep < eplim; ep++ )
            {
                fscanf_s( file, "%f, %f\n", &e1, &e2 );
                printf("%5.3g, %5.3g\n", e1, e2 );
                report[fi][ep][0] += sqrt(e1);
                report[fi][ep][1] += sqrt(e2);
            }

            fclose( file );
            printf( "\n\n" );
        }
    }

    sprintf_s( buf, 256, "%s.txt", str.c_str() );
    fopen_s( &file, buf, "wt" );
    for ( int ep = 0; ep < eplim; ep++ )
    {
        for ( int fi = 0; fi < 5; fi++ )
        {
            fprintf_s( file, "%7.5g, %7.5g, ", 
                report[fi][ep][0] / (10.0 * imax), 
                report[fi][ep][1] / (10.0 * imax) );
        }
        fprintf_s( file, "\n" );
    }

    fclose( file );
}


int main(int argc, char* argv[])
{
    //process_diag_mdiff( std::string("89i_32h_n1") );
    //process_diag_mdiff( std::string("33i_16h_n1") );
    //process_diag_mdiff( std::string("33i_2h_n1") );
    //process_diag_mdiff( std::string("33i_12h_n1") );
    //process_diag_mdiff( std::string("33i_32h_n1") );
    process_diag_conv( std::string("33i_2h_n1") );
    process_diag_conv( std::string("33i_12h_n1") );
    process_diag_conv( std::string("33i_32h_n1") );
    //process_diag_conv( std::string("89i_32h_n1") );
    return 0;
}

